package MT::Plugin::SKR::MostlyUsedMenu;
use strict;
use MT 4;
use MT::PluginData;

use vars qw( $MYNAME $VERSION );
($MYNAME) = __PACKAGE__ =~ /.+::(\w+)/;
$VERSION = '0.01';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
        id => $MYNAME,
        key => $MYNAME,
        name => $MYNAME,
        version => $VERSION,
        author_name => 'SKYARC System Co.,Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
        doc_link => 'http://www.skyarc.co.jp/engineerblog/entry/mostlyusedmenu.html',
        description => <<HTMLHEREDOC,
<__trans phrase="Add the menu in which the mostly used items are shown.">
HTMLHEREDOC
        l10n_class => $MYNAME. '::L10N',
        registry => {
            applications => {
                cms => {
                    menus => {
                        $MYNAME => {
                            label => 'Mostly Used Menu',
                            order => 0,
                        },
                    },
                    methods => {
                        add_recent_items => \&_hdlr_add_recent_items,
                    },
                },
            },
            callbacks => {
                'MT::App::CMS::template_source.header' => \&template_source_header,
                'pre_run' => \&_hdlr_pre_run,
                'MT::Author::pre_remove' => \&_hdlr_pre_remove_author,
            },
        },
});
MT->add_plugin( $plugin );

sub instance { $plugin; }



### Methods - add_recent_items
sub _hdlr_add_recent_items {
    my ($app) = @_;

	my $refer = $ENV{HTTP_REFERER}
        or return 0;
    $refer =~ s!^https?://[^/]+!!;

    my $cfg = MT->config;
    my $mt_cgi = ($cfg->AdminCGIPath || $cfg->CGIPath). $cfg->AdminScript;
    $mt_cgi =~ s!^https?://[^/]+!!;

    $refer =~ m!^\Q$mt_cgi\E!
        or return 0;
    my $user = $app->user
        or return 0;

	my $data = load_plugindata (key_name ($user->id))
        || {};
    if (defined $data->{$refer}) {
        $data->{$refer}{count}++;
    }
    else {
	    my $title = $app->param('title') || $app->translate ('Untitled');
	    $title =~ s/(.+?)\s*\|\s*.+/$1/;
        $data->{$refer} = {
		    title => $title,
			count => 1,
		};
    }
	save_plugindata (key_name ($user->id), $data);

	1;
}

### Callbacks - MT::App::CMS::template_source.header
sub template_source_header {
    my ($cb, $app, $tmpl) = @_;

    my $old = quotemeta (<<'HTMLHEREDOC');
        <mt:if name="top_nav_loop">
HTMLHEREDOC
    my $new = (<<'HTMLHEREDOC');
<script type="text/javascript">
jQuery.get (
    '<$MTCGIPath$><$MTAdminScript$>',
    { __mode: 'add_recent_items', title: document.title } //, function (d) { alert (d) }
);
</script>
HTMLHEREDOC
    $$tmpl =~ s!($old)!$1$new!;
}

### Callbacks - pre_run
sub _hdlr_pre_run {
    my ($cb, $app) = @_;

    my $user = $app->user or return;
    my $data = load_plugindata (key_name ($user->id))
        || {};
    my @urls = sort { $data->{$b}{count} <=> $data->{$a}{count} } keys %$data;
	pop @urls while 5 < @urls;

    my $order = 1;
    if (5.0 <= $MT::VERSION) {
        foreach (@urls) {
            $app->registry ('applications', 'cms', 'menus',
            "$MYNAME:$order", {
                label => $data->{$_}{title},
                order => $order++,
                link => $_,
            });
        }
    }
    else {
        my $menus = $app->registry ('applications', 'cms', 'menus');
        foreach (@urls) {
            $menus->{"$MYNAME:$order"} = {
                label => $data->{$_}{title},
                order => $order++,
                link => $_,
            };
        }
        $app->registry ('applications', 'cms', 'menus', $menus);
    }
}

### Callbacks - MT::Author::pre_remove
sub _hdlr_pre_remove_author {
    my ($cb, $obj) = @_; remove_plugindata (key_name ($obj->id));
}

########################################################################
sub key_name { 'author_id::'. $_[0]; }

sub save_plugindata {
    my ($key, $data_ref) = @_;
    my $pd = MT::PluginData->load ({ plugin => &instance->id, key => $key });
    if (!$pd) {
        $pd = MT::PluginData->new;
        $pd->plugin(&instance->id);
        $pd->key ($key);
    }
    $pd->data ($data_ref);
    $pd->save;
}

sub load_plugindata {
    my ($key) = @_;
    my $pd = MT::PluginData->load ({ plugin => &instance->id, key => $key })
        or return undef;
    $pd->data;
}

sub remove_plugindata {
    my ($key) = @_;
    my $pd = MT::PluginData->load ({ plugin => &instance->id, key => $key })
        or return undef;# there is no entry all along
    $pd->remove;
}

1;